MissionMinder = MissionMinder or {};
MissionMinder.ptr = GetBuildInfo():match('6[.]1[.][0-9]+')

local addon = MissionMinder;
addon.name = "MissionMinder"; --internal name; purposefully non-localized
addon.version = GetAddOnMetadata(addon.name, "Version");
addon.options = {};
local options = addon.options;

local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");

function MissionMinder.debug(format, ...)
	--[===[@alpha@
	print("|cffcccccc[DEBUG] " .. string.format(format, ...));
	--@end-alpha@]===]
end

local DEBUG = MissionMinder.debug;

---------------------------------------------
-- OPTIONS
---------------------------------------------

local LISTVIEW_ENABLED = "ListViewEnabled"
local AUTO_OPEN_REWARDS = "AutoOpenRewards";
local AUTO_CLOSE = "AutoClose";
local SKIP_FIGHT_ANIMATIONS = "SkipFightAnimations";
local QUICK_LOOT = "QuickLoot";
local SHOW_FOLLOWER_XP_GAINS = "ShowFollowerXPGains";
local SHOW_FOLLOWER_COUNTER_TOTALS = "ShowFollowerCounterTotals";
local IGNORE_XPGAINS_TRAITS = "IgnoreXPGainTraits";

local INPROGRESS_MISSIONS = "InProgressMissions";

MissionMinderGlobals = {};
local MissionMinderGlobals_DEFAULT = {
	[INPROGRESS_MISSIONS] = {}
};

MissionMinderOptions = {};
local MissionMinderOptions_DEFAULT = {
	[LISTVIEW_ENABLED] = true,
	[AUTO_OPEN_REWARDS] = false,
	[AUTO_CLOSE] = false,
	[SKIP_FIGHT_ANIMATIONS] = false,
	[QUICK_LOOT] = false,
	[SHOW_FOLLOWER_XP_GAINS] = false,
	[SHOW_FOLLOWER_COUNTER_TOTALS] = true,
	[IGNORE_XPGAINS_TRAITS] = true
};

function options.ListViewEnabled()
	return MissionMinderOptions[LISTVIEW_ENABLED];
end

function options.AutoLootRewards()
	return MissionMinderOptions[AUTO_OPEN_REWARDS];
end

function options.AutoClose()
	return MissionMinderOptions[AUTO_CLOSE];
end

function options.SkipFightAnimations()
	return MissionMinderOptions[SKIP_FIGHT_ANIMATIONS];
end

function options.QuickLoot()
	return options.ListViewEnabled() and MissionMinderOptions[QUICK_LOOT];
end

function options.ShowFollowerXPGains()
	return options.QuickLoot() and MissionMinderOptions[SHOW_FOLLOWER_XP_GAINS];
end

function options.ShowFollowerCounterTotals()
	return MissionMinderOptions[SHOW_FOLLOWER_COUNTER_TOTALS];
end

function options.IgnoreXPGainTraits()
	return MissionMinderOptions[IGNORE_XPGAINS_TRAITS];
end


---------------------------------------------
-- OPTIONS PANE
---------------------------------------------

local config = LibStub("AceConfig-3.0")
local dialog = LibStub("AceConfigDialog-3.0")

local function createOptions()
	local function getValue(info)
		return MissionMinderOptions[info.arg];
	end

	local function setValue(info, value)
		MissionMinderOptions[info.arg] = value;
	end

	local options = {
		name = "MissionMinder " .. addon.version,
		desc = L["Manage garrison missions in a more streamline, less-invasive manner"],
		type = "group",
		args = {
			desc = {
				order = 2,
				type = "description",
				name = L["Manage garrison missions in a more streamline, less-invasive manner"],
				width = "full"
			},
			emptySpace = {
				order = 3,
				type = "description",
				name = " ",
				width = "full"
			},
			listview = {
				order = 10,
				type = "group",
				inline = true,
				name = L["List View"],
				args = {
					description = {
						order = 1,
						type = "description",
						name = L["View completed missions as a list from the \"In Progress\" tab"],
						width = "full",
					},
					spacer = {
						order = 2,
						type = "description",
						name = " ",
						width = "full",
					},
					listviewEnabled = {
						order = 10,
						name = L["Enabled"],
						desc = L["View completed missions as a list from the \"In Progress\" tab"],
						type = "toggle",
						width = "full",
						set = setValue,
						get = getValue,
						arg = LISTVIEW_ENABLED
					},
					quickLoot = {
						order = 25,
						name = L["Quick loot"],
						desc = L["Disable the completion frame entirely, skip all the fanfare, and just give me the loot already!"],
						disabled = function() return not options.ListViewEnabled(); end,
						type = "toggle",
						width = "full",
						set = setValue,
						get = getValue,
						arg = QUICK_LOOT
					},
					showXP = {
						order = 30,
						name = L["Show follower XP gains"],
						desc = L["Show messages regarding individual follower XP gains"],
						disabled = function() return not options.ListViewEnabled() or not options.QuickLoot(); end,
						type = "toggle",
						width = "full",
						set = setValue,
						get = getValue,
						arg = SHOW_FOLLOWER_XP_GAINS
					}
				}
			},
			general = {
				order = 15,
				type = "group",
				inline = true,
				disabled = options.QuickLoot,
				name = L["General Options"],
				args = {
					skipFightAnimations = {
						order = 5,
						name = L["Skip fight animations"],
						desc = L["Skip all fight animations and go straight to rewards"],
						type = "toggle",
						width = "full",
						set = setValue,
						get = getValue,
						arg = SKIP_FIGHT_ANIMATIONS
					},
					autoOpenRewards = {
						order = 10,
						name = L["Automatically open rewards chest"],
						desc = L["Remove the need to click on the rewards chest to recieve the rewards"],
						type = "toggle",
						width = "full",
						set = setValue,
						get = getValue,
						arg = AUTO_OPEN_REWARDS
					},
					autoClose = {
						order = 15,
						name = L["Automatically close on loot"],
						desc = L["Automatically close the completion dialog if all actions have been completed for a mission"],
						type = "toggle",
						width = "full",
						set = setValue,
						get = getValue,
						arg = AUTO_CLOSE
					}
				}
			},
			planner = {
				order = 20,
				type = "group",
				inline = true,
				name = L["Planner"],
				args = {
					showFollowerCounterTotals = {
						order = 5,
						name = L["Show active follower counter totals"],
						desc = L["Easily track how many of each counter you currently have active"],
						type = "toggle",
						width = "full",
						set = function(info, value)
							setValue(info, value);
							if ( GarrisonMissionFrame:IsShown() and GarrisonMissionFrame.FollowerTab:IsShown()) then
								if(value and not MissionMinderActiveFollowerCounters:IsShown() ) then
									MissionMinderActiveFollowerCounters:Show();
									if ( MissionMinder.ptr ) then
										GarrisonThreatCountersFrame:Hide();
									end
								elseif(not value and MissionMinderActiveFollowerCounters:IsShown()) then
									MissionMinderActiveFollowerCounters:Hide();
									if ( MissionMinder.ptr ) then
										GarrisonThreatCountersFrame:Show();
									end
								end
							end
						end,
						get = getValue,
						arg = SHOW_FOLLOWER_COUNTER_TOTALS
					},
					ignoreXPGainTraits = {
						order = 10,
						name = L["Ignore XP increase traits"],
						desc = L["Do not show XP increase traits on missions that do not reward Bonus XP"],
						type = "toggle",
						width = "full",
						set = setValue,
						get = getValue,
						arg = IGNORE_XPGAINS_TRAITS
					}
				}
			}
		}
	};

	config:RegisterOptionsTable("MissionMinderOptions", options);
	return dialog:AddToBlizOptions("MissionMinderOptions", "MissionMinder");
end

local function ShowConfigureTooltip(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	GameTooltip:SetText(L["Configure MissionMinder"]);
end

local frame = createOptions();
addon.options.frame = frame;


function frame.OnEvent(self, event, arg1)
	if event == "ADDON_LOADED" and arg1 == addon.name then
		frame:UnregisterEvent("ADDON_LOADED");

		-- SavedVariables have been loaded, rewrite to include any new or missing values
		for key,value in pairs(MissionMinderOptions_DEFAULT) do
			if MissionMinderOptions[key] == nil then
				MissionMinderOptions[key] = value;
			end
		end
		for key,value in pairs(MissionMinderGlobals_DEFAULT) do
			if MissionMinderGlobals[key] == nil then
				MissionMinderGlobals[key] = value;
			end
		end

		-- Rewrite to remove any deprecated settings
		for key,value in pairs(MissionMinderOptions) do
			if MissionMinderOptions_DEFAULT[key] == nil then
				MissionMinderOptions[key] = nil;
			end
		end
		for key,value in pairs(MissionMinderGlobals) do
			if MissionMinderGlobals_DEFAULT[key] == nil then
				MissionMinderGlobals[key] = nil;
			end
		end


		GarrisonMissionFrameConfigureButton:HookScript("OnEnter", ShowConfigureTooltip)
		GarrisonLandingPageConfigureButton:HookScript("OnEnter", ShowConfigureTooltip)
 	end
end
frame:SetScript("OnEvent", frame.OnEvent);
frame:RegisterEvent("ADDON_LOADED");
