local Lib = MissionMinder;
Lib.callbacks = Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

local options = Lib.options;
local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");
local DEBUG = Lib.debug;

Lib.tracker = {};
local module = Lib.tracker;


local MINUTES = 60
local HOURS = 60 * MINUTES
local function secondsToString(seconds)
	if ( seconds >= HOURS ) then
		return string.format(INT_GENERAL_DURATION_HOURS, seconds / HOURS)
	elseif ( seconds >= MINUTES ) then
		return string.format(INT_GENERAL_DURATION_MIN, seconds / MINUTES)
	else
		return string.format(INT_GENERAL_DURATION_SEC, seconds)
	end
end

local function trackMission(event, missionID, ...)
	local player = GetUnitName("player") .. "-" .. GetRealmName()
	local now = GetTime()

	local inProgressMissions = MissionMinderGlobals.InProgressMissions
	if ( not inProgressMissions[player]) then
		inProgressMissions[player] = {}
	end

	local totalTimeInSeconds = select(2, C_Garrison.GetPartyMissionInfo(missionID));
	if (totalTimeInSeconds) then
		inProgressMissions[player][missionID] = now + totalTimeInSeconds
	end
end

local function untrackMission(event, missionID)
	local player = GetUnitName("player") .. "-" .. GetRealmName()

	local inProgressMissions = MissionMinderGlobals.InProgressMissions
	if ( inProgressMissions[player]) then
		inProgressMissions[player][missionID] = nil
	end
end

local function orderTrackedMissions(track1, track2)
	if ( track1.completed ~= track2.completed ) then
		return track1.completed > track2.completed
	elseif ( track1.lowest ~= track2.lowest ) then
		return track1.lowest < track2.lowest
	else
		return strcmputf8i(track1.name, track2.name) < 0;
	end
end


local function showTrackMissionTooltip()
	local now = GetTime();

	local tracking = {}
	local inProgressMissions = MissionMinderGlobals.InProgressMissions
	for player, missions in pairs(inProgressMissions) do
		local lowest = nil;
		local completed = 0

		for missionID, endDate in pairs(missions) do
			if ( endDate <= now ) then
				lowest = 0;
				completed = completed + 1
			elseif ( not lowest or endDate - now < lowest ) then
				lowest = endDate - now;
			end
		end

		if ( lowest ) then
			local tbl = {}
			tbl.name = player
			tbl.lowest = lowest
			tbl.completed = completed
			tinsert(tracking, tbl)
		end
	end

	if ( #tracking > 0 ) then
		GameTooltip:AddLine(" ");
		GameTooltip:AddLine(L["Account Missions"]);
		table.sort(tracking, orderTrackedMissions);
		for i = 1, #tracking do
			if ( tracking[i].completed > 0 ) then
				GameTooltip:AddDoubleLine(tracking[i].name, tracking[i].compelted .. " " .. L["completed"], 1, 1, 1, 0, 1, 0);
			else
				GameTooltip:AddDoubleLine(tracking[i].name, secondsToString(tracking[i].lowest), 1, 1, 1, 0.7, 0.7, 0.7);
			end
		end;
		GameTooltip:Show()
	end
end

local function initialize()
	DEBUG("   |cffffffffTracker|r initialized");

	-- Track mission progress across multiple characters
	Lib.RegisterCallback("tracker", "MISSION_STARTED", trackMission)
	Lib.RegisterCallback("tracker", "MISSION_COMPLETED", untrackMission)
	GarrisonLandingPageMinimapButton:HookScript("OnEnter", showTrackMissionTooltip)
end

local events = Lib.events
events:RegisterEvent("tracker", "ADDON_LOADED", initialize);
